@@ -1,11 +1,10 @@
 package Mojolicious::Plugin::TtRenderer::Engine;
-{
-  $Mojolicious::Plugin::TtRenderer::Engine::VERSION = '1.43_01';
+BEGIN {
+  $Mojolicious::Plugin::TtRenderer::Engine::VERSION = '1.20';
 }
 
 use warnings;
 use strict;
-use v5.10;
 
 use base 'Mojo::Base';
 
@@ -14,14 +13,11 @@ use File::Spec ();
 use Mojo::ByteStream 'b';
 use Template ();
 use Cwd qw/abs_path/;
-use Scalar::Util 'weaken';
-use POSIX ':errno_h';
 
 __PACKAGE__->attr('tt');
 
 sub build {
     my $self = shift->SUPER::new(@_);
-    weaken($self->{app});
     $self->_init(@_);
     return sub { $self->_render(@_) }
 }
@@ -32,37 +28,24 @@ sub _init {
 
     #$Template::Parser::DEBUG = 1;
 
-    my $dir;
     my $app = delete $args{mojo} || delete $args{app};
-    if($dir=$args{cache_dir}) {
 
-      if($app && substr($dir,0,1) ne '/') {
-        $dir=$app->home->rel_dir($dir);
-      }
-    }
+    my $dir = $app && $app->home->rel_dir('tmp/ctpl');
 
     # TODO
     #   take and process options :-)
 
-    my @renderer_paths = $app ? map { abs_path($_) } grep { -d $_ } @{ $app->renderer->paths } : ();
-    push @renderer_paths, 'templates';
-
     my %config = (
-        INCLUDE_PATH => \@renderer_paths,
-        COMPILE_EXT  => '.ttc',
-        UNICODE      => 1,
-        ENCODING     => 'utf-8',
-        CACHE_SIZE   => 128,
-        RELATIVE     => 1,
+        ($app ? (INCLUDE_PATH => abs_path($app->home->rel_dir('templates'))) : ()),
+        COMPILE_EXT => '.ttc',
+        COMPILE_DIR => ($dir || abs_path(File::Spec->tmpdir)),
+        UNICODE     => 1,
+        ENCODING    => 'utf-8',
+        CACHE_SIZE  => 128,
+        RELATIVE    => 1,
         %{$args{template_options} || {}},
     );
 
-    $config{COMPILE_DIR} //= $dir || do {
-      my $tmpdir = File::Spec->catdir(File::Spec->tmpdir, "ttr$<");
-      mkdir $tmpdir unless -d $tmpdir;
-      $tmpdir;
-    };
-
     $config{LOAD_TEMPLATES} =
       [Mojolicious::Plugin::TtRenderer::Provider->new(%config, renderer => $app->renderer)]
       unless $config{LOAD_TEMPLATES};
@@ -91,32 +74,22 @@ sub _render {
     $$output = '';
 
     my @params = ({%{$c->stash}, c => $c, h => $helper}, $output, {binmode => ':utf8'});
-    my $provider = $self->tt->{SERVICE}->{CONTEXT}->{LOAD_TEMPLATES}->[0];
-    $provider->options($options);
-    $provider->ctx($c);
+    $self->tt->{SERVICE}->{CONTEXT}->{LOAD_TEMPLATES}->[0]->ctx($c);
 
-    my $ok = do {
-        if (defined $inline) {
-            $self->tt->process(\$inline, @params);
-        }
-        else {
-            my @ret = $provider->fetch($t);
-
-            if (not defined $ret[1]) {
-                $self->tt->process($ret[0], @params);
-            }
-            elsif (not defined $ret[0]) { # not found
-                return 0;
-            }
-            else { # error
-                return 0 if $! == ENOENT && (not ref $ret[0]); # not found when not blessed exception
-                die $ret[0];
-            }
-        }
-    };
+    my $ok = $self->tt->process(defined $inline ? \$inline : $t, @params);
 
     # Error
-    die $self->tt->error unless $ok;
+    unless ($ok) {
+
+        my $e = Mojo::Exception->new(
+            $self->tt->error.'',
+            $self->tt->service->process(defined $inline ? \$inline : $t));
+        $$output = '';
+        $c->app->log->error(qq/Template error in "$t": $e/);
+        $c->render_exception($e);
+        $self->tt->error('');
+        return 0;
+    }
 
     return 1;
 }
@@ -160,7 +133,6 @@ use strict;
 use warnings;
 
 use base 'Template::Provider';
-use Scalar::Util 'weaken';
 
 sub new {
     my $class = shift;
@@ -170,52 +142,35 @@ sub new {
 
     my $self = $class->SUPER::new(%params);
     $self->renderer($renderer);
-    weaken($self->{renderer});
     return $self;
 }
 
 sub renderer      { @_ > 1 ? $_[0]->{renderer}      = $_[1] : $_[0]->{renderer} }
 sub ctx           { @_ > 1 ? $_[0]->{ctx}           = $_[1] : $_[0]->{ctx} }
-sub options       { @_ > 1 ? $_[0]->{options}       = $_[1] : $_[0]->{options} }
 
-sub _template_modified {
-    my($self, $template) = @_;
-    $self->SUPER::_template_modified($template) || $template =~ /^templates(?:\/|\\)/;
-}
+sub _template_modified {1}
 
 sub _template_content {
     my $self = shift;
     my ($path) = @_;
 
-    my $options = delete $self->{options};
-    
     # Convert backslashes to forward slashes to make inline templates work on Windows
     $path =~ s/\\/\//g;
     my ($t) = ($path =~ m{templates\/(.*)$});
-    
+
     if (-r $path) {
         return $self->SUPER::_template_content(@_);
     }
 
-    my $data;
-    my $error = '';
-
     # Try DATA section
-    if(defined $options) {
-        $data = $self->renderer->get_data_template($options);
-    } else {
-        my $loader = Mojo::Loader->new;
-        foreach my $class (@{ $self->renderer->classes }) {
-            $data = $loader->data($class, $t);
-            last if $data;
-        }
+    elsif (my $d = $self->renderer->get_data_template($self->ctx, $t)) {
+        return wantarray ? ($d, '', time) : $d;
     }
 
-    unless($data) {
-        $data = '';
-        $error = "$path: not found";
-    }
-    return wantarray ? ($data, $error, time) : $data;
+    my $data = '';
+    my $error = "$path: not found";
+    my $mod_date = time;
+    return wantarray ? ($data, $error, $mod_date) : $data;
 }
 
 1;
@@ -232,27 +187,27 @@ Mojolicious::Plugin::TtRenderer::Engine - Template Toolkit renderer for Mojo
 
 Add the handler:
 
- sub startup {
-     ...
- 
-     # Via mojolicious plugin
-     $self->plugin(tt_renderer => {template_options => {FILTERS => [ ... ]}});
- 
-     # Or manually
-     use Mojolicious::Plugin::TtRenderer::Engine;
- 
-     my $tt = Mojolicious::Plugin::TtRenderer::Engine->build(
-         mojo => $self,
-         template_options => {
-             PROCESS  => 'tpl/wrapper',
-             FILTERS  => [ ... ],
-             UNICODE  => 1,
-             ENCODING => 'UTF-8',
-         }
-     );
+    sub startup {
+        ...
 
-     $self->renderer->add_handler( tt => $tt );
- }
+        # Via mojolicious plugin
+        $self->plugin(tt_renderer => {template_options => {FILTERS => [ ... ]}});
+
+        # Or manually
+        use Mojolicious::Plugin::TtRenderer::Engine;
+
+        my $tt = Mojolicious::Plugin::TtRenderer::Engine->build(
+            mojo => $self,
+            template_options => {
+                PROCESS  => 'tpl/wrapper',
+                FILTERS  => [ ... ],
+                UNICODE  => 1,
+                ENCODING => 'UTF-8',
+            }
+        );
+
+        $self->renderer->add_handler( tt => $tt );
+    }
 
 Template parameter are taken from C< $c->stash >.
 
@@ -262,20 +217,20 @@ The template file for C<"example/welcome"> would be C<"templates/welcome.html.tt
 
 When template file is not available rendering from C<__DATA__> is attempted.
 
- __DATA__
+    __DATA__
 
- @@ welcome.html.tt
- Welcome, [% user.name %]!
+    @@ welcome.html.tt
+    Welcome, [% user.name %]!
 
 Inline template is also supported:
 
- $self->render(inline => '[% 1 + 1 %]', handler => 'tt');
+    $self->render(inline => '[% 1 + 1 %]', handler => 'tt');
 
 =head1 HELPERS
 
 Helpers are exported automatically under C<h> namespace.
 
- [% h.url_for('index') %]
+    [% h.url_for('index') %]
 
 =head1 METHODS
 
@@ -293,42 +248,34 @@ object. When used the INCLUDE_PATH will be set to
 
 =item template_options
 
-A hash reference of options that are passed to Template->new().  Note that if you
-specify an C<INCLUDE_PATH> through this option it will remove the DATA section
-templates from your path.  A better way to specify an C<INCLUDE_PATH> if you also
-want to use DATA section templates it by manipulting the L<Mojolicious::Renderer>
-path.
-
-=item cache_dir
-
-Absolute or relative dir to your app home, to cache processed versions of your
-templates. Will default to a temp-dir if not set.
+A hash reference of options that are passed to Template->new().
 
 =back
 
-=head1 SEE ALSO
-
-L<Mojolicious::Plugin::TtRenderer::Engine>, 
-L<Mojolicious>, 
-L<Mojolicious::Guides>, 
-L<http://mojolicious.org>.
-
 =head1 AUTHOR
 
-Current maintainer: Graham Ollis C<< <plicease@cpan.org> >>
+Ask Bjørn Hansen, C<< <ask at develooper.com> >>
 
-Original author: Ask Bjørn Hansen, C<< <ask at develooper.com> >>
+=head1 TODO
+
+   * Better support non-Mojolicious frameworks
+   * Move the default template cache directory?
+   * Better way to pass parameters to the templates? (stash)
+   * More sophisticated default search path?
 
 =head1 BUGS
 
-Please report any bugs or feature requests to the project's github issue tracker
-L<https://github.com/abh/mojox-renderer-tt/issues?state=open>.
+Please report any bugs or feature requests to C<bug-mojox-renderer-tt at rt.cpan.org>,
+or through the web interface at
+L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=MojoX-Renderer-TT>.  I will be
+notified, and then you'll automatically be notified of progress on your bug as I
+make changes.
 
 =head1 SUPPORT
 
 You can find documentation for this module with the perldoc command.
 
- perldoc Mojolicious::Plugin::TtRenderer::Engine
+    perldoc Mojolicious::Plugin::TtRenderer::Engine
 
 You can also look for information at:
 
@@ -341,6 +288,10 @@ L<git://git.develooper.com/MojoX-Renderer-TT.git>
 
 L<http://github.com/abh/mojox-renderer-tt/>
 
+=item * RT: CPAN's request tracker
+
+L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=MojoX-Renderer-TT>
+
 =item * CPAN Ratings
 
 L<http://cpanratings.perl.org/d/MojoX-Renderer-TT>
@@ -351,12 +302,13 @@ L<http://search.cpan.org/dist/MojoX-Renderer-TT/>
 
 =back
 
+=head1 ACKNOWLEDGEMENTS
+
+
 =head1 COPYRIGHT & LICENSE
 
 Copyright 2008-2010 Ask Bjørn Hansen, all rights reserved.
 
-Copyright 2012 Graham Ollis.
-
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
 
